

#include <string.h>

#include "discoord.h"


/* **********************************************************
********************************************************** */
DISPDU_detonate_type::DISPDU_detonate_type() {

   header.type = DIS_PDU_TYPE_DETONATE;
   header.family = DIS_PDU_FAMILY_ENTITY_INFO_INTERACT;
   header.size = DIS_PDU_SIZE_BASE_DETONATE;
   fire_site_id = dis_site_id;
   fire_application_id = dis_application_id;
   ammo_site_id = dis_site_id;
   ammo_application_id = dis_application_id;
   event_site_id = dis_site_id;
   event_application_id = dis_application_id;
}


/* **********************************************************
   creates and initalizes detonate_pdu
********************************************************** */
void DISPDU_detonate_type::setup(DISPDU_fire_type *fire,
   int fire_id, DIS_id_type *target, int ammo_id, int event_id,
   DIS_burst_descriptor_type *burst, double *pos, double *vel,
   double *target_local_pos, int det_result, unsigned int timer) {

//   header.size += asdf - no articulated parts for now
   header.time = timer;

   copyarray3(location, pos);
   copyarray3(velocity, (float)vel);
   copyarray3(location_target_coords, (float)target_local_pos);
   detonation_result = det_result;
   articulation_count = 0; // asdf - no articulated parts for now

   if (fire) {
      fire_entity_id = fire->fire_entity_id;

      target_site_id = fire->target_site_id;
      target_application_id = fire->target_application_id;
      target_entity_id = fire->target_entity_id;

      ammo_entity_id = fire->ammo_entity_id;

      event_site_id = fire->event_site_id;
      event_application_id = fire->event_application_id;
      event_entity_id = fire->event_entity_id;

      memcpy(&burst_data, &fire->burst_data, sizeof(DIS_burst_descriptor_type));
      return;
   }
   
   fire_entity_id = fire_id;

   target_site_id = target->site;
   target_application_id = target->application;
   target_entity_id = target->entity;

   if (ammo_id)
      ammo_entity_id = ammo_id;
   else {
      ammo_entity_id = 0;
      ammo_application_id = 0;
      ammo_site_id = 0;
   }

   event_entity_id = event_id;

   memcpy(&burst_data, burst, sizeof(DIS_burst_descriptor_type));
}


/* **********************************************************
********************************************************** */
void DISPDU_detonate_type::build(buffer_type *buffer) {

   sfile sbuffer;
   int i;

   sbuffer.dest();

   header.build(&sbuffer);

   sbuffer.sput_ushort(fire_site_id);
   sbuffer.sput_ushort(fire_application_id);
   sbuffer.sput_ushort(fire_entity_id);

   sbuffer.sput_ushort(target_site_id);
   sbuffer.sput_ushort(target_application_id);
   sbuffer.sput_ushort(target_entity_id);

   sbuffer.sput_ushort(ammo_site_id);
   sbuffer.sput_ushort(ammo_application_id);
   sbuffer.sput_ushort(ammo_entity_id);

   sbuffer.sput_ushort(event_site_id);
   sbuffer.sput_ushort(event_application_id);
   sbuffer.sput_ushort(event_entity_id);

   sbuffer.sput_float(velocity[0]);
   sbuffer.sput_float(velocity[1]);
   sbuffer.sput_float(velocity[2]);

   sbuffer.sput_double(location[0]);
   sbuffer.sput_double(location[1]);
   sbuffer.sput_double(location[2]);

   burst_data.build(&sbuffer);

   sbuffer.sput_float(location_target_coords[0]);
   sbuffer.sput_float(location_target_coords[1]);
   sbuffer.sput_float(location_target_coords[2]);

   sbuffer.sput_char(detonation_result);
   sbuffer.sput_char(articulation_count);
   sbuffer.sput_char(padd[0]);
   sbuffer.sput_char(padd[1]);

   for (i=0; i<articulation_count; i++) {
//asdf put 128 bit articulation record in
   }

   buffer->buffercpy((char *)sbuffer.data, sbuffer.vsize);
}


/* **********************************************************
********************************************************** */
void DISPDU_detonate_type::extract(buffer_type *buffer) {

   sfile sbuffer;
   int i;

   sbuffer.scan_data(buffer);

   header.extract(&sbuffer);

   fire_site_id = sbuffer.scan_ushort();
   fire_application_id = sbuffer.scan_ushort();
   fire_entity_id = sbuffer.scan_ushort();

   target_site_id = sbuffer.scan_ushort();
   target_application_id = sbuffer.scan_ushort();
   target_entity_id = sbuffer.scan_ushort();

   ammo_site_id = sbuffer.scan_ushort();
   ammo_application_id = sbuffer.scan_ushort();
   ammo_entity_id = sbuffer.scan_ushort();

   event_site_id = sbuffer.scan_ushort();
   event_application_id = sbuffer.scan_ushort();
   event_entity_id = sbuffer.scan_ushort();

   velocity[0] = sbuffer.scan_float();
   velocity[1] = sbuffer.scan_float();
   velocity[2] = sbuffer.scan_float();

   location[0] = sbuffer.scan_double();
   location[1] = sbuffer.scan_double();
   location[2] = sbuffer.scan_double();

   burst_data.extract(&sbuffer);

   location_target_coords[0] = sbuffer.scan_float();
   location_target_coords[1] = sbuffer.scan_float();
   location_target_coords[2] = sbuffer.scan_float();

   detonation_result = sbuffer.scan_char();
   articulation_count = sbuffer.scan_char();
   sbuffer.skip(2);

   for (i=0; i<articulation_count; i++) {
//asdf put 128 bit articulation record in
   }

}
